/*
** krb5.c for phpns in /tmp/ns
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Sun Apr  6 10:24:21 2008 corentin chary
** Last update Sun Apr  6 15:35:52 2008 corentin chary
*/

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <krb5.h>
#include <gssapi.h>

#include "krb.h"

#define NETSOUL_SERVICE_NAME	"host@ns-server.epitech.net"

static krb5_error_code	krb_init_pass(krb5_context context,
				      krb5_principal principal,
				      krb5_ccache ccache,
				      const char *pass);
static int	krb_init(const char *login, const char *pass);
static void	krb_init_context(gss_buffer_desc *otoken);

static krb5_error_code		krb_init_pass(krb5_context context,
					      krb5_principal principal,
					      krb5_ccache ccache,
					      const char *pass)
{
   krb5_error_code		ret;
   krb5_get_init_creds_opt	opt;
   krb5_creds			cred;

   memset(&cred, 0, sizeof(cred));
   krb5_get_init_creds_opt_init(&opt);
   ret = krb5_get_init_creds_password(context,
                                      &cred,
                                      principal,
                                      (char *)pass,
                                      krb5_prompter_posix,
                                      NULL,
                                      0,
                                      NULL,
                                      &opt);
   if (ret == KRB5_LIBOS_PWDINTR ||
       ret == KRB5KRB_AP_ERR_MODIFIED ||
       ret == KRB5KRB_AP_ERR_MODIFIED)
      return (-1);
   else if (ret)
      return (-1);
   if (krb5_cc_initialize(context, ccache, cred.client))
      return (-1);
   if (krb5_cc_store_cred(context, ccache, &cred))
      return (-1);
   return (0);
}

static int		krb_init(const char *login, const char *pass)
{
   krb5_error_code	ret;
   krb5_context		context;
   krb5_ccache		ccache;
   krb5_principal	principal;

   if (krb5_init_context(&context))
      return (-1);
   if (krb5_parse_name(context, login, &principal))
      return (-1);
   if (krb5_cc_default(context, &ccache))
      return (-1);
   ret = krb_init_pass(context, principal, ccache, pass);
   krb5_cc_close(context, ccache);
   krb5_free_principal(context, principal);
   krb5_free_context(context);
   return (ret);
}

static void		krb_init_context(gss_buffer_desc *otoken)
{
  OM_uint32		min;
  gss_buffer_t		itoken = GSS_C_NO_BUFFER;
  gss_ctx_id_t		ctx;
  gss_buffer_desc	buf;
  gss_name_t		gss_name;

  ctx = NULL;
  buf.value = strdup(NETSOUL_SERVICE_NAME);
  buf.length = strlen(buf.value) + 1;
  gss_import_name(&min, &buf, GSS_C_NT_HOSTBASED_SERVICE, &gss_name);
  gss_init_sec_context(&min,
		       GSS_C_NO_CREDENTIAL,
		       &ctx,
		       gss_name,
		       GSS_C_NO_OID,
		       GSS_C_CONF_FLAG,
		       0,
		       GSS_C_NO_CHANNEL_BINDINGS,
		       itoken,
		       NULL,
		       otoken,
		       NULL,
		       NULL);
}

gss_buffer_desc		*krb_get_token(const char *login, const char *pass)
{
  gss_buffer_desc	*otoken;

  otoken = calloc(sizeof (*otoken), 1);
  if (!otoken)
    return (NULL);
  krb_init_context(otoken);
  if (!otoken->value)
    {
      krb_init(login, pass);
      krb_init_context(otoken);
    }
  if (otoken->value)
    return (otoken);
  free (otoken);
  return (NULL);
}
