/*
** php_ns.c for phpns in /tmp/ns
** 
** Made by corentin chary
** Login   <chary_c@epitech.net>
** 
** Started on  Sun Apr  6 10:22:33 2008 corentin chary
** Last update Sun Apr  6 16:37:39 2008 corentin chary
*/

/* $Id: header,v 1.16.2.1.2.1 2007/01/01 19:32:09 iliaa Exp $ */

#include <signal.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ns.h"
#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_ns.h"

static int le_link;

ZEND_DECLARE_MODULE_GLOBALS(ns)

/* True global resources - no need for thread safety here */
static int le_ns;

/* {{{ ns_functions[]
 *
 * Every user visible function must have an entry in ns_functions[].
 */
zend_function_entry ns_functions[] = {
	PHP_FE(ns_connect, NULL)
	PHP_FE(ns_close, NULL)
	PHP_FE(ns_state, NULL)
	PHP_FE(ns_user_cmd_msg_user, NULL)
	PHP_FE(ns_user_cmd_who, NULL)
	PHP_FE(ns_user_cmd_watch_log_user, NULL)
	PHP_FE(ns_receive_cmd, NULL)
	{NULL, NULL, NULL}	/* Must be the last line in ns_functions[] */
};
/* }}} */

/* {{{ ns_module_entry
 */
zend_module_entry ns_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"ns",
	ns_functions,
	PHP_MINIT(ns),
	PHP_MSHUTDOWN(ns),
	PHP_RINIT(ns),
	PHP_RSHUTDOWN(ns),
	PHP_MINFO(ns),
#if ZEND_MODULE_API_NO >= 20010901
	"0.1",
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_NS
ZEND_GET_MODULE(ns)
#endif

/* {{{ PHP_INI
 */
PHP_INI_BEGIN()
STD_PHP_INI_ENTRY("ns.default_port", "4242", PHP_INI_ALL, OnUpdateLong,
				  default_port, zend_ns_globals, ns_globals)
STD_PHP_INI_ENTRY("ns.default_group", "bocal",  PHP_INI_ALL, OnUpdateString,
				  default_group, zend_ns_globals, ns_globals)
STD_PHP_INI_ENTRY("ns.default_location", "phpnetsoul",  PHP_INI_ALL, OnUpdateString,
				  default_location, zend_ns_globals, ns_globals)
STD_PHP_INI_ENTRY("ns.default_system", "Linux",  PHP_INI_ALL, OnUpdateString,
				  default_system, zend_ns_globals, ns_globals)
PHP_INI_END()
/* }}} */

/* {{{ php_ns_init_globals
 */
static void php_ns_init_globals(zend_ns_globals *ns_globals)
{
	(void) ns_globals;
}
/* }}} */

/* {{{ _close_mysql_link
 */
static void _close_ns_link(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	t_ns	*link;
	void	(*handler) (int); 

	link = (t_ns *)rsrc->ptr;
	handler = signal(SIGPIPE, SIG_IGN);
	ns_close(link);
	signal(SIGPIPE, handler);
}
/* }}} */


/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(ns)
{
	REGISTER_INI_ENTRIES();
	le_link = zend_register_list_destructors_ex(_close_ns_link, NULL, "ns link", module_number);
	Z_TYPE(ns_module_entry) = type;
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(ns)
{
	UNREGISTER_INI_ENTRIES();
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request start */
/* {{{ PHP_RINIT_FUNCTION
 */
PHP_RINIT_FUNCTION(ns)
{
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request end */
/* {{{ PHP_RSHUTDOWN_FUNCTION
 */
PHP_RSHUTDOWN_FUNCTION(ns)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(ns)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "ns support", "enabled");
	php_info_print_table_end();

	DISPLAY_INI_ENTRIES();
}
/* }}} */

static void		ns_set_opt(t_ns *ns)
{
	char		*p;

	ns->port = -1;
	if (p = strchr(ns->host, ':'))
		{
			ns->port = strtol(p + 1, NULL, 10);
			*p = '\0';
		}
	if (ns->port < 0)
		ns->port = ns_globals.default_port;
	if (!ns->location)
		ns->location = ns_globals.default_location;
	if (!ns->group)
		ns->group = ns_globals.default_group;
	if (!ns->system)
		ns->system = ns_globals.default_system;
	ns->host = strdup(ns->host);
	ns->user = strdup(ns->user);
	ns->location = ns_encode(ns->location);
	ns->group = ns_encode(ns->group);
	ns->system = ns_encode(ns->system);
}

/* {{{ proto resource ns_connect(string hostname[:port] , string username, string password [, string location [, string group [, string system]]])
   Opens a connection to a Netsoul Server */
PHP_FUNCTION(ns_connect)
{
	char		*pass;
	t_ns		*ns;
	int			dummy;

	ns = calloc(sizeof (*ns), 1);
	if (!ns)
		RETURN_FALSE;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s!s!s!s!s!s!",
							  &ns->host, &dummy, &ns->user, &dummy, &pass, &dummy,
							  &ns->location, &dummy, &ns->group, &dummy,
							  &ns->system, &dummy) == FAILURE)
		RETURN_FALSE;
	ns_set_opt(ns);
	if (ns_connect(ns))
		{
			php_error_docref(NULL TSRMLS_CC, E_WARNING,
							 "Can't connect to %s", ns->host);
			ns_close(ns);
			RETURN_FALSE;
		}
	if (ns_auth(ns, pass))
		{
			php_error_docref(NULL TSRMLS_CC, E_WARNING,
							 "Can't auth, check your username");
			ns_close(ns);
			RETURN_FALSE;
		}
	ns_state(ns, "actif");
	ZEND_REGISTER_RESOURCE(return_value, ns, le_link);
	zend_list_addref(Z_LVAL_P(return_value));
}
/* }}} */

/* {{{ proto bool ns_close(int link_identifier)
   Close a Netsoul connection */
PHP_FUNCTION(ns_close)
{
	zval		**ns_link;
	int			id;
	t_ns		*ns;

	ns_link = NULL;
	if (ZEND_NUM_ARGS() == 1)
		{
			if (zend_get_parameters_ex(1, &ns_link) == FAILURE)
				RETURN_FALSE;
			id = -1;
		}
	else
		WRONG_PARAM_COUNT;
	ZEND_FETCH_RESOURCE(ns, t_ns *, ns_link, id, "NS-Link", le_link);
	zend_list_delete(Z_RESVAL_PP(ns_link));
	RETURN_TRUE;
}
/* }}} */

/* {{{ proto bool ns_state(int link_identifier, string state)
   Change the status of a Netsoul connection */
PHP_FUNCTION(ns_state)
{
	zval		**ns_link;
	zval		**state;
	t_ns		*ns;
	int			id;

	ns_link = NULL;
	state = NULL;
	id = -1;
    if (ZEND_NUM_ARGS() == 2)
		{
			if (zend_get_parameters_ex(2, &ns_link, &state)==FAILURE)
				RETURN_FALSE;
		}
	else
		WRONG_PARAM_COUNT;
	ZEND_FETCH_RESOURCE(ns, t_ns *, ns_link, id, "NS-Link", le_link);
    convert_to_string_ex(state);
	ns_state(ns, Z_STRVAL_PP(state));
	RETURN_TRUE;
}
/* }}} */

/* {{{ proto bool ns_user_cmd_msg_user(int link_identifier, string user, string msg)
   Send a message to a Netsoul contact */
PHP_FUNCTION(ns_user_cmd_msg_user)
{
	zval		**ns_link;
	zval		**msg;
	zval		**user;
	t_ns		*ns;
	int			id;

	ns_link = NULL;
	user = NULL;
	msg = NULL;
	id = -1;
    if (ZEND_NUM_ARGS() == 3)
		{
			if (zend_get_parameters_ex(3, &ns_link, &user, &msg)==FAILURE)
				RETURN_FALSE;
		}
	else
		WRONG_PARAM_COUNT;
	ZEND_FETCH_RESOURCE(ns, t_ns *, ns_link, id, "NS-Link", le_link);
    convert_to_string_ex(user);
	convert_to_string_ex(msg);
	ns_user_cmd_msg_user(ns, Z_STRVAL_PP(user), Z_STRVAL_PP(msg));
	RETURN_TRUE;
}
/* }}} */

/* {{{ proto bool ns_user_cmd_who(int link_identifier, string who)
   Send a message to a Netsoul contact */
PHP_FUNCTION(ns_user_cmd_who)
{
	zval		**ns_link;
	zval		**user;
	t_ns		*ns;
	int			id;

	ns_link = NULL;
	user = NULL;
	id = -1;
    if (ZEND_NUM_ARGS() == 2)
		{
			if (zend_get_parameters_ex(2, &ns_link, &user) == FAILURE)
				RETURN_FALSE;
		}
	else
		WRONG_PARAM_COUNT;
	ZEND_FETCH_RESOURCE(ns, t_ns *, ns_link, id, "NS-Link", le_link);
    convert_to_string_ex(user);
	ns_user_cmd_who(ns, Z_STRVAL_PP(user));
	RETURN_TRUE;
}
/* }}} */

/* {{{ proto bool ns_user_cmd_watch_log_user(int link_identifier, string user)
   Send a message to a Netsoul contact */
PHP_FUNCTION(ns_user_cmd_watch_log_user)
{
	zval		**ns_link;
	zval		**user;
	t_ns		*ns;
	int			id;

	ns_link = NULL;
	user = NULL;
	id = -1;
    if (ZEND_NUM_ARGS() == 2)
		{
			if (zend_get_parameters_ex(2, &ns_link, &user) == FAILURE)
				RETURN_FALSE;
		}
	else
		WRONG_PARAM_COUNT;
	ZEND_FETCH_RESOURCE(ns, t_ns *, ns_link, id, "NS-Link", le_link);
    convert_to_string_ex(user);
	ns_user_cmd_watch_log_user(ns, Z_STRVAL_PP(user));
	RETURN_TRUE;
}
/* }}} */

/* {{{ proto string ns_receive_cmd(int link_identifier, int timeout)
   Send a message to a Netsoul contact */
PHP_FUNCTION(ns_receive_cmd)
{
	zval		**ns_link;
	zval		**timeout;
	long		id;
	t_ns		*ns;
	char		*line;

	ns_link = NULL;
	timeout = NULL;
	id = -1;
    if (ZEND_NUM_ARGS() == 2)
		{
			if (zend_get_parameters_ex(2, &ns_link, &timeout) == FAILURE)
				RETURN_FALSE;
		}
	else
		WRONG_PARAM_COUNT;
	ZEND_FETCH_RESOURCE(ns, t_ns *, ns_link, id, "NS-Link", le_link);
    convert_to_long_ex(timeout);
	line = ns_receive_cmd(ns, Z_LVAL_PP(timeout));
	if (line)
		{
			RETURN_STRING(line, 1);
		}
	else
		RETURN_FALSE;
}
/* }}} */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
